<?php

declare(strict_types=1);

namespace Drupal\Tests\visitors\Unit\Form;

use Drupal\Tests\UnitTestCase;
use Drupal\visitors\Form\DateFilter;
use Symfony\Component\DependencyInjection\ContainerBuilder;

/**
 * Date filter form test.
 *
 * @group visitors
 */
class DateFilterTest extends UnitTestCase {

  /**
   * The form.
   *
   * @var \Drupal\visitors\Form\DateFilter
   */
  protected $form;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $messenger;

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $state;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $requestStack;

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $database;

  /**
   * The date time.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $dateTime;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $container = new ContainerBuilder();

    $string_translation = $this->getStringTranslationStub();
    $container->set('string_translation', $string_translation);

    $date_formatter = $this->createMock('Drupal\Core\Datetime\DateFormatterInterface');
    $container->set('date.formatter', $date_formatter);

    $this->dateTime = $this->createMock('Drupal\Component\Datetime\TimeInterface');
    $container->set('datetime.time', $this->dateTime);

    $this->database = $this->createMock('Drupal\Core\Database\Connection');
    $container->set('database', $this->database);

    $language = $this->createMock('Drupal\Core\Language\LanguageInterface');
    $language->method('getId')->willReturn('en');
    $language_manager = $this->createMock('Drupal\Core\Language\LanguageManagerInterface');
    $language_manager->method('getCurrentLanguage')->willReturn($language);
    $container->set('language_manager', $language_manager);

    \Drupal::setContainer($container);

    $this->form = DateFilter::create($container);
  }

  /**
   * Test the form id.
   */
  public function testGetFormId() {
    $this->assertEquals('visitors_date_filter_form', $this->form->getFormId());
  }

  /**
   * Test the form build.
   */
  public function testBuildForm() {
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form = $this->form->buildForm([], $form_state);
    $this->assertIsArray($form);
    $this->assertCount(2, $form);
    $this->assertArrayHasKey('visitors_date_filter', $form);
    $this->assertArrayHasKey('submit', $form);
  }

  /**
   * Test the form validation. Invalid from date.
   */
  public function testValidateFormInvalidFrom() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn('2019-01-01');
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn('2019-01-01');
    $form = [
      'from' => [],
      'to' => [],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn($date_from, $date_to);

    $this->form->validateForm($form, $form_state);
  }

  /**
   * Test the form validation. Invalid To date.
   */
  public function testValidateFormInvalidTo() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn('2019-01-01');
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2019);
    $form = [
      'from' => [
        'month' => 1,
        'day' => 1,
        'year' => 2019,
      ],
      'to' => [],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn($date_from, $date_to);

    $this->form->validateForm($form, $form_state);
  }

  /**
   * Test the form validation.
   */
  public function testValidateForm() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2019);
    $form = [
      'from' => [
        'month' => 1,
        'day' => 1,
        'year' => 2019,
      ],
      'to' => [
        'month' => 1,
        'day' => 1,
        'year' => 2020,
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn($date_from, $date_to);

    $this->form->validateForm($form, $form_state);
  }

  /**
   * Test the form validation. From date greater than To date.
   */
  public function testValidateFormToGreaterThanFrom() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2019);
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $form = [
      'from' => [
        'month' => 1,
        'day' => 1,
        'year' => 2020,
      ],
      'to' => [
        'month' => 1,
        'day' => 1,
        'year' => 2019,
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn($date_from, $date_to);
    $form_state->expects($this->once())
      ->method('setError')
      ->with($form['from'], 'The specified date is invalid.');
    $this->form->validateForm($form, $form_state);
  }

  /**
   * Test the form validation. From date negative.
   */
  public function testValidateFormNegativeFrom() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2019);
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 1492);
    $form = [
      'from' => [
        'month' => 1,
        'day' => 1,
        'year' => 1492,
      ],
      'to' => [
        'month' => 1,
        'day' => 1,
        'year' => 2019,
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn($date_from, $date_to);
    $form_state->expects($this->once())
      ->method('setError')
      ->with($form['from'], 'The specified date is invalid.');
    $this->form->validateForm($form, $form_state);
  }

  /**
   * Test the form validation. To date negative.
   */
  public function testValidateFormNegativeTo() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 1492);
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $form = [
      'from' => [
        'month' => 1,
        'day' => 1,
        'year' => 2020,
      ],
      'to' => [
        'month' => 1,
        'day' => 1,
        'year' => 1492,
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn($date_from, $date_to);
    $form_state->expects($this->once())
      ->method('setError')
      ->with($form['to'], 'The specified date is invalid.');
    $this->form->validateForm($form, $form_state);
  }

  /**
   * Test formProcessDatelist. Month empty.
   */
  public function testFormProcessDatelistMonthEmpty() {
    $query_result = $this->createMock('Drupal\Core\Database\StatementInterface');
    $query_result->expects($this->once())
      ->method('fetchField')
      ->willReturn(1708619104);
    $select = $this->createMock('\Drupal\Core\Database\Query\Select');
    $select->expects($this->once())
      ->method('addExpression')
      ->willReturnSelf();
    $select->expects($this->once())
      ->method('execute')
      ->willReturn($query_result);
    $this->database->expects($this->once())
      ->method('select')
      ->willReturn($select);
    $date = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $element = [
      '#default_value' => $date,
      '#required' => FALSE,
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn([
        'from' => [
          'month' => 1,
          'day' => 1,
          'year' => 2019,
        ],
        'to' => [
          'month' => 1,
          'day' => 1,
          'year' => 2020,
        ],
      ]);
    $this->form->formProcessDatelist($element, $form_state);
  }

  /**
   * Test formProcessDatelist. Day empty.
   */
  public function testFormProcessDatelistDayEmpty() {
    $query_result = $this->createMock('Drupal\Core\Database\StatementInterface');
    $query_result->expects($this->once())
      ->method('fetchField')
      ->willReturn(1708619104);
    $select = $this->createMock('\Drupal\Core\Database\Query\Select');
    $select->expects($this->once())
      ->method('addExpression')
      ->willReturnSelf();
    $select->expects($this->once())
      ->method('execute')
      ->willReturn($query_result);
    $this->database->expects($this->once())
      ->method('select')
      ->willReturn($select);
    $date = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $element = [
      '#value' => [
        'month' => 1,
      ],
      '#default_value' => $date,
      '#required' => FALSE,
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn([
        'from' => [
          'month' => 1,
          'day' => 1,
          'year' => 2019,
        ],
        'to' => [
          'month' => 1,
          'day' => 1,
          'year' => 2020,
        ],
      ]);
    $this->form->formProcessDatelist($element, $form_state);
  }

  /**
   * Test formProcessDatelist. Year empty.
   */
  public function testFormProcessDatelistYearEmpty() {
    $query_result = $this->createMock('Drupal\Core\Database\StatementInterface');
    $query_result->expects($this->once())
      ->method('fetchField')
      ->willReturn(1708619104);
    $select = $this->createMock('\Drupal\Core\Database\Query\Select');
    $select->expects($this->once())
      ->method('addExpression')
      ->willReturnSelf();
    $select->expects($this->once())
      ->method('execute')
      ->willReturn($query_result);
    $this->database->expects($this->once())
      ->method('select')
      ->willReturn($select);
    $date = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $element = [
      '#value' => [
        'month' => 1,
        'day' => 1,
      ],
      '#default_value' => $date,
      '#required' => FALSE,
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn([
        'from' => [
          'month' => 1,
          'day' => 1,
          'year' => 2019,
        ],
        'to' => [
          'month' => 1,
          'day' => 1,
          'year' => 2020,
        ],
      ]);
    $this->form->formProcessDatelist($element, $form_state);
  }

  /**
   * Test formProcessDatelist.
   */
  public function testFormProcessDatelist() {
    $query_result = $this->createMock('Drupal\Core\Database\StatementInterface');
    $query_result->expects($this->once())
      ->method('fetchField')
      ->willReturn(1708619104);
    $select = $this->createMock('\Drupal\Core\Database\Query\Select');
    $select->expects($this->once())
      ->method('addExpression')
      ->willReturnSelf();
    $select->expects($this->once())
      ->method('execute')
      ->willReturn($query_result);
    $this->database->expects($this->once())
      ->method('select')
      ->willReturn($select);
    $date = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date->expects($this->exactly(3))
      ->method('format')
      ->willReturn(1, 1, 2020);
    $element = [
      'month' => 1,
      'day' => 1,
      'year' => 2020,
      '#default_value' => $date,
      '#required' => TRUE,
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValue')
      ->willReturn([
        'from' => [
          'month' => 1,
          'day' => 1,
          'year' => 2019,
        ],
        'to' => [
          'month' => 1,
          'day' => 1,
          'year' => 2020,
        ],
      ]);
    $this->form->formProcessDatelist($element, $form_state);
  }

  /**
   * Test datelistValueCallback.
   */
  public function testDatelistValueCallbackTrue() {
    $element = [
      '#value' => [
        'month' => 1,
        'day' => 1,
        'year' => 2020,
      ],
    ];
    $form_state = [];
    $result = $this->form->datelistValueCallback($element, TRUE, $form_state);
    $this->assertIsArray($result);
    $this->assertCount(3, $result);
  }

  /**
   * Test datelistValueCallback.
   */
  public function testDatelistValueCallbackFalse() {
    $element = [
      '#value' => [
        'month' => 1,
        'day' => 1,
        'year' => 2020,
      ],
    ];
    $form_state = [];
    $result = $this->form->datelistValueCallback($element, TRUE, $form_state);
    $this->assertIsArray($result);
    $this->assertCount(3, $result);
  }

  /**
   * Test datelistValidate.
   */
  public function testDatelistValidateInputExists() {
    $element = [
      '#parents' => [
        'to',
        'from',
      ],
      '#value' => [
        'month' => 1,
        'day' => 1,
        'year' => 2020,
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([
        'from' => [
          'month' => 1,
          'day' => 1,
          'year' => 2019,
        ],
        'to' => [
          'month' => 1,
          'day' => 1,
          'year' => 2020,
        ],
      ]);

    $result = $this->form->datelistValidate($element, $form_state);
    $this->assertNull($result);
  }

  /**
   * Test datelistValidate.
   */
  public function testDatelistValidateYearEmpty() {
    $element = [
      '#parents' => [],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([]);
    $form_state->expects($this->once())
      ->method('setError');
    $this->form->datelistValidate($element, $form_state);
  }

  /**
   * Test datelistValidate. Month empty.
   */
  public function testDatelistValidateMonthEmpty() {
    $element = [
      '#parents' => [
        'to',
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([
        'to' => ['year' => 2020],
      ]);
    $form_state->expects($this->once())
      ->method('setError');
    $this->form->datelistValidate($element, $form_state);
  }

  /**
   * Test datelistValidate. Day empty.
   */
  public function testDatelistValidateDayEmpty() {
    $element = [
      '#parents' => [
        'to',
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([
        'to' => ['year' => 2020, 'month' => 1],
      ]);
    $form_state->expects($this->once())
      ->method('setError');
    $this->form->datelistValidate($element, $form_state);
  }

  /**
   * Test datelistValidate. Invalid date.
   */
  public function testDatelistValidateInvalidDate() {
    $element = [
      '#parents' => [
        'to',
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([
        'to' => ['year' => 2020, 'month' => 1, 'day' => 32],
      ]);
    $form_state->expects($this->once())
      ->method('setError');
    $this->form->datelistValidate($element, $form_state);
  }

  /**
   * Test datelistValidate. Valid date.
   */
  public function testDatelistValidateValidDate() {
    $element = [
      '#parents' => [
        'to',
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([
        'to' => ['year' => 2020, 'month' => 1, 'day' => 1],
      ]);
    $result = $this->form->datelistValidate($element, $form_state);
    $this->assertNull($result);
  }

  /**
   * Test datelistValidate. Valid date with extra. unable to test the last line.
   *
   * @todo Handle exceptions when there are invalid times.
   */
  public function ftestDatelistValidateValidDateExtra() {
    $element = [
      '#parents' => [
        'to',
      ],
    ];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->method('getValues')
      ->willReturn([
        'to' => [
          'year' => 2020,
          'month' => 1,
          'day' => 1,
          'hour' => 'extra',
        ],
      ]);
    $this->form->datelistValidate($element, $form_state);
  }

}
