<?php

declare(strict_types=1);

namespace Drupal\Tests\visitors\Unit\Form;

use Drupal\Tests\UnitTestCase;
use Drupal\user\RoleInterface;
use Drupal\visitors\Form\Settings;
use Prophecy\PhpUnit\ProphecyTrait;
use Symfony\Component\DependencyInjection\ContainerBuilder;

/**
 * Tests settings form.
 *
 * @group visitors
 */
class SettingsTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The form under test.
   *
   * @var \Drupal\visitors\Form\Settings
   */
  protected $form;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $currentUser;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleHandler;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $httpClient;

  /**
   * The extension theme list.
   *
   * @var \Drupal\Core\Extension\ThemeExtensionList|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $extensionThemeList;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configFactory;

  /**
   * The request stack.
   *
   * @var \Drupal\Core\Session\SessionConfigurationInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $sessionConfiguration;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $requestStack;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $container = new ContainerBuilder();

    $string_translation = $this->getStringTranslationStub();
    $container->set('string_translation', $string_translation);

    $this->currentUser = $this->createMock('Drupal\Core\Session\AccountInterface');
    $container->set('current_user', $this->currentUser);

    $this->moduleHandler = $this->createMock('Drupal\Core\Extension\ModuleHandlerInterface');
    $container->set('module_handler', $this->moduleHandler);

    $this->httpClient = $this->createMock('GuzzleHttp\Client');
    $container->set('http_client', $this->httpClient);

    $session = $this->createMock('Symfony\Component\HttpFoundation\Session\SessionInterface');
    $container->set('session', $session);

    $this->sessionConfiguration = $this->createMock('Drupal\Core\Session\SessionConfigurationInterface');
    $container->set('session_configuration', $this->sessionConfiguration);

    $this->extensionThemeList = $this->createMock('Drupal\Core\Extension\ThemeExtensionList');
    $container->set('extension.list.theme', $this->extensionThemeList);

    $this->entityTypeManager = $this->createMock('Drupal\Core\Entity\EntityTypeManagerInterface');
    $container->set('entity_type.manager', $this->entityTypeManager);

    $this->configFactory = $this->createMock('Drupal\Core\Config\ConfigFactoryInterface');
    $container->set('config.factory', $this->configFactory);

    $this->requestStack = $this->createMock('Symfony\Component\HttpFoundation\RequestStack');
    $container->set('request_stack', $this->requestStack);

    $unrouted_url_assembler = $this->createMock('Drupal\Core\Utility\UnroutedUrlAssemblerInterface');
    $container->set('unrouted_url_assembler', $unrouted_url_assembler);

    $url_generator = $this->createMock('Drupal\Core\Routing\UrlGeneratorInterface');
    $container->set('url_generator', $url_generator);

    $messenger = $this->createMock('Drupal\Core\Messenger\MessengerInterface');
    $container->set('messenger', $messenger);

    \Drupal::setContainer($container);

    $this->form = Settings::create($container);
  }

  /**
   * Test the getFormId method.
   */
  public function testGetFormId() {
    $this->assertEquals('visitors_admin_settings', $this->form->getFormId());
  }

  /**
   * Test the buildForm method.
   */
  public function testBuildForm() {
    $user_role_storage = $this->createMock('Drupal\Core\Entity\EntityStorageInterface');
    $this->entityTypeManager->expects($this->exactly(2))
      ->method('getStorage')
      ->with('user_role')
      ->willReturn($user_role_storage);
    $anonymous_role = $this->createMock('Drupal\user\RoleInterface');
    $anonymous_role->expects($this->exactly(1))
      ->method('id')
      ->willReturn(RoleInterface::ANONYMOUS_ID);
    $anonymous_role->expects($this->exactly(2))
      ->method('label')
      ->willReturn('Anonymous');
    $authenticate_role = $this->createMock('Drupal\user\RoleInterface');
    $authenticate_role->expects($this->exactly(1))
      ->method('id')
      ->willReturn(RoleInterface::AUTHENTICATED_ID);
    $authenticate_role->expects($this->exactly(2))
      ->method('label')
      ->willReturn('Authenticated');
    $administrator_role = $this->createMock('Drupal\user\RoleInterface');
    $administrator_role->expects($this->exactly(1))
      ->method('id')
      ->willReturn('administrator');
    $administrator_role->expects($this->exactly(2))
      ->method('label')
      ->willReturn('Administrator');
    $user_roles = [$anonymous_role, $authenticate_role, $administrator_role];
    $user_role_storage->expects($this->exactly(2))
      ->method('loadMultiple')
      ->willReturn($user_roles);

    $config = $this->createMock('Drupal\Core\Config\Config');
    $config
      ->expects($this->exactly(22))
      ->method('get');
    $system_config = $this->createMock('Drupal\Core\Config\Config');
    $system_config->expects($this->exactly(2))
      ->method('get')
      ->willReturn('bartik', 'claro');
    $this->configFactory->expects($this->once())
      ->method('get')
      ->with('system.theme')
      ->willReturn($system_config);

    $this->configFactory->expects($this->once())
      ->method('getEditable')
      ->with('visitors.config')
      ->willReturn($config);

    $bartik_theme = $this->createMock('\Drupal\Core\Extension\Extension');
    $bartik_theme->info = ['name' => 'bartik'];
    // @phpstan-ignore-next-line
    $bartik_theme->status = 1;

    $claro_theme = $this->createMock('\Drupal\Core\Extension\Extension');
    $claro_theme->info = ['name' => 'claro'];
    // @phpstan-ignore-next-line
    $claro_theme->status = 1;
    $this->extensionThemeList->expects($this->once())
      ->method('getList')
      ->willReturn(['bartik' => $bartik_theme, 'claro' => $claro_theme]);

    $request = $this->createMock('Symfony\Component\HttpFoundation\Request');
    $request_stack = $this->createMock('Symfony\Component\HttpFoundation\RequestStack');
    $request_stack->expects($this->exactly(2))
      ->method('getCurrentRequest')
      ->willReturn($request);
    $this->form->setRequestStack($request_stack);

    $this->sessionConfiguration->expects($this->once())
      ->method('getOptions')
      ->willReturn(['cookie_domain' => 'visitors.ddev.site']);

    $node_definition = $this->createMock('Drupal\Core\Entity\ContentEntityType');
    $node_type_definition = $this->createMock('Drupal\Core\Config\Entity\ConfigEntityType');

    $this->entityTypeManager->expects($this->once())
      ->method('getDefinitions')
      ->willReturn([$node_definition, $node_type_definition]);

    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form = $this->form->buildForm([], $form_state);

    $this->assertIsArray($form);
    $this->assertArrayHasKey('actions', $form);
    $this->assertArrayHasKey('#theme', $form);
    $this->assertArrayHasKey('theme', $form);
    $this->assertArrayHasKey('tracking_scope', $form);
    $this->assertArrayHasKey('tracking', $form);
    $this->assertArrayHasKey('performance_track', $form);
    $this->assertArrayHasKey('entity', $form);
    $this->assertArrayHasKey('advanced', $form);
  }

  /**
   * Test the buildForm method with different options.
   */
  public function testBuildFormOtherSettings() {
    $user_role_storage = $this->createMock('Drupal\Core\Entity\EntityStorageInterface');
    $this->entityTypeManager->expects($this->exactly(2))
      ->method('getStorage')
      ->with('user_role')
      ->willReturn($user_role_storage);
    $anonymous_role = $this->createMock('Drupal\user\RoleInterface');
    $anonymous_role->expects($this->exactly(1))
      ->method('id')
      ->willReturn(RoleInterface::ANONYMOUS_ID);
    $anonymous_role->expects($this->exactly(2))
      ->method('label')
      ->willReturn('Anonymous');
    $authenticate_role = $this->createMock('Drupal\user\RoleInterface');
    $authenticate_role->expects($this->exactly(1))
      ->method('id')
      ->willReturn(RoleInterface::AUTHENTICATED_ID);
    $authenticate_role->expects($this->exactly(2))
      ->method('label')
      ->willReturn('Authenticated');
    $administrator_role = $this->createMock('Drupal\user\RoleInterface');
    $administrator_role->expects($this->exactly(1))
      ->method('id')
      ->willReturn('administrator');
    $administrator_role->expects($this->exactly(2))
      ->method('label')
      ->willReturn('Administrator');
    $user_roles = [$anonymous_role, $authenticate_role, $administrator_role];
    $user_role_storage->expects($this->exactly(2))
      ->method('loadMultiple')
      ->willReturn($user_roles);

    $config = $this->createMock('Drupal\Core\Config\Config');
    $config
      ->expects($this->exactly(22))
      ->method('get')
      ->willReturn(
        'admin',
        'admin',
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP,
        Settings::VISIBILITY_REQUEST_PATH_MODE_PHP
      );
    $system_config = $this->createMock('Drupal\Core\Config\Config');
    $system_config->expects($this->exactly(2))
      ->method('get')
      ->willReturn('bartik', 'claro');
    $this->configFactory->expects($this->once())
      ->method('get')
      ->with('system.theme')
      ->willReturn($system_config);

    $this->configFactory->expects($this->once())
      ->method('getEditable')
      ->with('visitors.config')
      ->willReturn($config);

    $bartik_theme = $this->createMock('\Drupal\Core\Extension\Extension');
    $bartik_theme->info = ['name' => 'bartik'];
    // @phpstan-ignore-next-line
    $bartik_theme->status = 1;

    $claro_theme = $this->createMock('\Drupal\Core\Extension\Extension');
    $claro_theme->info = ['name' => 'claro'];
    // @phpstan-ignore-next-line
    $claro_theme->status = 1;
    $this->extensionThemeList->expects($this->once())
      ->method('getList')
      ->willReturn(['bartik' => $bartik_theme, 'claro' => $claro_theme]);

    $request = $this->createMock('Symfony\Component\HttpFoundation\Request');
    $request_stack = $this->createMock('Symfony\Component\HttpFoundation\RequestStack');
    $request_stack->expects($this->exactly(2))
      ->method('getCurrentRequest')
      ->willReturn($request);
    $this->form->setRequestStack($request_stack);

    $this->sessionConfiguration->expects($this->once())
      ->method('getOptions')
      ->willReturn(['cookie_domain' => 'ddev.site']);

    $node_definition = $this->createMock('Drupal\Core\Entity\ContentEntityType');
    $node_type_definition = $this->createMock('Drupal\Core\Config\Entity\ConfigEntityType');

    $this->entityTypeManager->expects($this->once())
      ->method('getDefinitions')
      ->willReturn([$node_definition, $node_type_definition]);

    $this->moduleHandler->expects($this->once())
      ->method('moduleExists')
      ->with('content_translation')
      ->willReturn(TRUE);

    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form = $this->form->buildForm([], $form_state);

    $this->assertIsArray($form);
    $this->assertArrayHasKey('actions', $form);
    $this->assertArrayHasKey('#theme', $form);
    $this->assertArrayHasKey('theme', $form);
    $this->assertArrayHasKey('tracking_scope', $form);
    $this->assertArrayHasKey('tracking', $form);
    $this->assertArrayHasKey('performance_track', $form);
    $this->assertArrayHasKey('entity', $form);
    $this->assertArrayHasKey('advanced', $form);
  }

  /**
   * Test the submitForm method.
   */
  public function testSubmitForm() {
    $form = [];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->expects($this->once())
      ->method('getValues')
      ->willReturn([
        'theme' => 'theme',
        'items_per_page' => 'items_per_page',
        'flush_log_timer' => 'flush_log_timer',
        'bot_retention_log' => 'bot_retention_log',
        'visitors_codesnippet_before' => 'visitors_codesnippet_before',
        'visitors_codesnippet_after' => 'visitors_codesnippet_after',
        'visitors_domain_mode' => 'visitors_domain_mode',
        'visitors_trackuserid' => 'visitors_trackuserid',
        'counter_enabled' => 'counter_enabled',
        'visitors_privacy_disablecookies' => 'visitors_privacy_disablecookies',
        'visitors_disable_tracking' => 'visitors_disable_tracking',
        'visitors_visibility_request_path_mode' => 'visitors_visibility_request_path_mode',
        'visitors_visibility_request_path_pages' => 'visitors_visibility_request_path_pages',
        'visitors_visibility_user_account_mode' => 'visitors_visibility_user_account_mode',
        'visitors_visibility_user_role_mode' => 'visitors_visibility_user_role_mode',
        'visitors_visibility_user_role_roles' => ['visitors_visibility_user_role_roles'],
        'visibility_exclude_user1' => 'visibility_exclude_user1',
        'status_codes_disabled' => ['status_codes_disabled'],
        'performance' => 'performance',
      ]);
    $form_state->expects($this->once())
      ->method('hasValue')
      ->with('visitors_translation_set')
      ->willReturn(TRUE);
    $form_state->expects($this->once())
      ->method('getValue')
      ->with('visitors_translation_set')
      ->willReturn('visitors_translation_set');

    $config = $this->createMock('Drupal\Core\Config\Config');
    $this->configFactory->expects($this->once())
      ->method('getEditable')
      ->with('visitors.config')
      ->willReturn($config);

    $config->method('set')
      ->willReturnSelf();
    $config->expects($this->exactly(2))
      ->method('save');

    $this->form->submitForm($form, $form_state);
  }

}
